特集2-1：SQL流条件比較

--***********************************
--
--○1－1．ウォーミングアップ：条件に応じて使う列を切り替える
--
--***********************************

CREATE TABLE CahngeCols
(year  INTEGER NOT NULL PRIMARY KEY,
 col_1 INTEGER NOT NULL,
 col_2 INTEGER NOT NULL);

INSERT INTO CahngeCols VALUES(2011,	10,	7);
INSERT INTO CahngeCols VALUES(2012,	20,	6);
INSERT INTO CahngeCols VALUES(2013,	30,	5);
INSERT INTO CahngeCols VALUES(2014,	40,	4);
INSERT INTO CahngeCols VALUES(2015,	50,	3);

--リスト1 列の切り替え（SELECT句で）
SELECT year,
       CASE WHEN year <= 2013 THEN col_1
            WHEN year >= 2014 THEN col_2
            ELSE NULL END AS col_3
  FROM CahngeCols;


--リスト2 WHERE句での利用
SELECT year
  FROM CahngeCols
 WHERE 4 <= CASE WHEN year <= 2013 THEN col_1
                 WHEN year >= 2014 THEN col_2
                 ELSE NULL END;


--***********************************
--
--○1－2．列の交換
--
--***********************************


CREATE TABLE Perm2
(cust_id  CHAR(3) PRIMARY KEY,
 item_1   VARCHAR(32)  NOT NULL,
 item_2   VARCHAR(32) NOT NULL);

INSERT INTO Perm2 VALUES('001', '時計', '浄水器');
INSERT INTO Perm2 VALUES('002', '携帯電話', '携帯電話');
INSERT INTO Perm2 VALUES('003', '浄水器', '時計');
INSERT INTO Perm2 VALUES('004', '携帯電話', '携帯電話');
INSERT INTO Perm2 VALUES('005', 'インク', 'メガネ');

--リスト3 組み合わせ⇒順列（重複行排除前）
SELECT CASE WHEN item_1 < item_2 THEN item_1
            ELSE item_2 END AS c1,
       CASE WHEN item_1 < item_2 THEN item_2
            ELSE item_1 END AS c2
  FROM Perm2;


--リスト4 組み合わせ⇒順列（重複行排除後）
SELECT DISTINCT
       CASE WHEN item_1 < item_2 THEN item_1
            ELSE item_2 END AS c1,
       CASE WHEN item_1 < item_2 THEN item_2
            ELSE item_1 END AS c2
  FROM Perm2;


CREATE TABLE Perm3
(cust_id  CHAR(3) PRIMARY KEY,
 item_1   VARCHAR(32)  NOT NULL,
 item_2   VARCHAR(32) NOT NULL,
 item_3   VARCHAR(32) NOT NULL);

INSERT INTO Perm3 VALUES('001', '時計', '浄水器', 'ティッシュ');
INSERT INTO Perm3 VALUES('002', 'ティッシュ', '浄水器', '時計');
INSERT INTO Perm3 VALUES('003', 'カレンダー', 'ノート', '時計');
INSERT INTO Perm3 VALUES('004', 'カレンダー', 'ノート', 'インク');
INSERT INTO Perm3 VALUES('005', '文庫本', 'ゲームソフト', 'メガネ');
INSERT INTO Perm3 VALUES('006', '文庫本', 'メガネ', 'ゲームソフト');

--リスト5 列持ち⇒行持ち
CREATE VIEW CustItems (cust_id, item) AS
SELECT cust_id, item_1
  FROM Perm3
UNION ALL
SELECT cust_id, item_2
  FROM Perm3
UNION ALL
SELECT cust_id, item_3
  FROM Perm3;

--リスト6 組み合わせ⇒順列（3列拡張版）
SELECT DISTINCT MAX(CI1.item) AS c1, 
       MAX(CI2.item) AS c2,
       MAX(CI3.item) AS c3
  FROM CustItems CI1
           INNER JOIN CustItems CI2
                   ON CI1.cust_id = CI2.cust_id
                  AND CI1.item < CI2.item
                 INNER JOIN CustItems CI3
                         ON CI2.cust_id = CI3.cust_id
                        AND CI2.item < CI3.item
 GROUP BY CI1.cust_id;



--***********************************
--
--○1－3．表頭の複雑な集計
--
--***********************************

CREATE TABLE Employees
(emp_id  CHAR(3) NOT NULL PRIMARY KEY,
 dept    VARCHAR(8) NOT NULL,
 sex     CHAR(2) NOT NULL,
 age     INTEGER NOT NULL,
 salary  INTEGER NOT NULL);

INSERT INTO Employees VALUES('001',	'製造',	'男',	32,	30);
INSERT INTO Employees VALUES('002',	'製造',	'男',	30,	29);
INSERT INTO Employees VALUES('003',	'製造',	'女',	23,	19);
INSERT INTO Employees VALUES('004',	'会計',	'男',	45,	35);
INSERT INTO Employees VALUES('005',	'会計',	'男',	50,	45);
INSERT INTO Employees VALUES('006',	'営業',	'女',	40,	50);
INSERT INTO Employees VALUES('007',	'営業',	'女',	42,	40);
INSERT INTO Employees VALUES('008',	'営業',	'男',	52,	38);
INSERT INTO Employees VALUES('009',	'営業',	'男',	34,	28);
INSERT INTO Employees VALUES('010',	'営業',	'女',	41,	25);
INSERT INTO Employees VALUES('011',	'人事',	'男',	29,	25);
INSERT INTO Employees VALUES('012',	'人事',	'女',	36,	29);


--リスト7 表頭：部署・性別、表側：年齢階級
SELECT dept,
       SUM(CASE WHEN age <= 30 AND sex = '男' THEN 1 ELSE 0 END) AS "若手（男）",
       SUM(CASE WHEN age <= 30 AND sex = '女' THEN 1 ELSE 0 END) AS "若手（女）",
       SUM(CASE WHEN age >= 31 AND sex = '男' THEN 1 ELSE 0 END) AS "ベテラン（男）",
       SUM(CASE WHEN age >= 31 AND sex = '女' THEN 1 ELSE 0 END) AS "ベテラン（女）"
  FROM Employees
 GROUP BY dept;


--リスト8 表頭：年齢階級・性別、表側：部署（小計・合計あり）
SELECT dept,
       COUNT(*),
       SUM(CASE WHEN age <= 30 THEN 1 ELSE 0 END) AS "若手（計）",
       SUM(CASE WHEN age <= 30 AND sex = '男' THEN 1 ELSE 0 END) AS "若手（男）",
       SUM(CASE WHEN age <= 30 AND sex = '女' THEN 1 ELSE 0 END) AS "若手（女）",
       SUM(CASE WHEN age >= 31 THEN 1 ELSE 0 END) AS "ベテラン（計）",
       SUM(CASE WHEN age >= 31 AND sex = '男' THEN 1 ELSE 0 END) AS "ベテラン（男）",
       SUM(CASE WHEN age >= 31 AND sex = '女' THEN 1 ELSE 0 END) AS "ベテラン（女）"
  FROM Employees
 GROUP BY dept;

--リスト9 全列をCOUNT関数で揃える
SELECT dept,
       COUNT(*),
       COUNT(CASE WHEN age <= 30 THEN 1 ELSE NULL END) AS "若手（計）",
       COUNT(CASE WHEN age <= 30 AND sex = '男' THEN 1 ELSE NULL END) AS "若手（男）",
       COUNT(CASE WHEN age <= 30 AND sex = '女' THEN 1 ELSE NULL END) AS "若手（女）",
       COUNT(CASE WHEN age >= 31 THEN 1 ELSE NULL END) AS "ベテラン（計）",
       COUNT(CASE WHEN age >= 31 AND sex = '男' THEN 1 ELSE NULL END) AS "ベテラン（男）",
       COUNT(CASE WHEN age >= 31 AND sex = '女' THEN 1 ELSE NULL END) AS "ベテラン（女）"
  FROM Employees
 GROUP BY dept;



--***********************************
--
--○1－4．集約関数の外でCASE式を使う
--
--***********************************

--リスト10 部署ごとの人数を選択する
SELECT dept,
       COUNT(*) AS cnt
  FROM Employees
 GROUP BY dept;


--リスト11 集約結果に対する条件設定：集約関数を引数にとる
SELECT dept,
       CASE WHEN COUNT(*) <= 2 THEN '2人以下'
            ELSE '3人以上' END AS cnt
  FROM Employees
 GROUP BY dept;

