特集2-2：SQL流行間比較

--***********************************
--
--○2－1．まずは基本：直近を求める
--
--***********************************

CREATE TABLE LoadSample
(sample_date DATE NOT NULL , 
 load_amt    INTEGER NOT NULL , 
   PRIMARY KEY (sample_date) ) ;

INSERT INTO LoadSample VALUES('2014-02-01', 1024);
INSERT INTO LoadSample VALUES('2014-02-02', 2366);
INSERT INTO LoadSample VALUES('2014-02-05', 2366);
INSERT INTO LoadSample VALUES('2014-02-07',  985);
INSERT INTO LoadSample VALUES('2014-02-08',  780);
INSERT INTO LoadSample VALUES('2014-02-12', 1000);


--直近（ウィンドウ関数：OracleのLAG関数）
SELECT sample_date,
       LAG(sample_date, 1) OVER (ORDER BY sample_date ASC) AS latest
  FROM LoadSample;

--リスト1 過去の直近を求める（ウィンドウ関数：MySQL以外で動作）
SELECT sample_date,
       MIN(sample_date) 
          OVER (ORDER BY sample_date ASC
                ROWS BETWEEN 1 PRECEDING AND 1 PRECEDING) AS latest
  FROM LoadSample;

--リスト2 直近（相関サブクエリ：実装非依存）
SELECT LS0.sample_date,
         (SELECT MAX(sample_date)
			FROM LoadSample LS1
		   WHERE LS1.sample_date < LS0.sample_date) AS latest
  FROM LoadSample LS0;

--リスト3 直近（自己結合：実装非依存）
SELECT LS0.sample_date AS cur_date,
       MAX(LS1.sample_date) AS latest
  FROM LoadSample LS0
           LEFT OUTER JOIN LoadSample LS1
             ON LS1.sample_date < LS0.sample_date
 GROUP BY LS0.sample_date;


--リスト4 説明用：非集約の状態で取ると
SELECT LS0.sample_date AS cur_date,
       LS1.sample_date AS latest
  FROM LoadSample LS0
           LEFT OUTER JOIN LoadSample LS1
             ON LS1.sample_date < LS0.sample_date;


--***********************************
--
--○2－2．直近、直近の1つ前、そのまた1つ前･･･
--
--***********************************

--リスト5 処理量も併せて表示する（Oracle以外で動作）
SELECT LS0.sample_date AS cur_date,
       MAX(LS0.load_amt) AS cur_load_amt,
       MAX(LS1.sample_date) AS latest,
       (SELECT MAX(load_amt)
          FROM LoadSample
         WHERE sample_date = MAX(LS1.sample_date)) AS latest_load_amt
  FROM LoadSample LS0
       LEFT OUTER JOIN LoadSample LS1
         ON LS1.sample_date < LS0.sample_date
 GROUP BY LS0.sample_date;

--リスト6 最大下界を求めるロジックをWHERE句に記述
SELECT LS0.sample_date AS cur_date,
       LS0.load_amt AS cur_load,
       LS1.sample_date AS latest,
       LS1.load_amt AS latest_load
  FROM LoadSample LS0
       LEFT OUTER JOIN LoadSample LS1
         ON LS1.sample_date = (SELECT MAX(sample_date)
                                 FROM LoadSample
                                WHERE sample_date < LS0.sample_date);

--リスト7 3つ前の日付まで出力する：ウィンドウ関数の利用
SELECT sample_date,
       MIN(sample_date) 
          OVER (ORDER BY sample_date ASC
                ROWS BETWEEN 1 PRECEDING AND 1 PRECEDING) AS latest,
       MIN(sample_date) 
          OVER (ORDER BY sample_date ASC
                ROWS BETWEEN 2 PRECEDING AND 2 PRECEDING) AS latest_2,
       MIN(sample_date) 
          OVER (ORDER BY sample_date ASC
                ROWS BETWEEN 3 PRECEDING AND 3 PRECEDING) AS latest_3
  FROM LoadSample;

--リスト8 3つ前の日付まで出力する：自己結合の利用
SELECT LS0.sample_date AS sample_date,
       MAX(LS1.sample_date) AS latest_1,
       MAX(LS2.sample_date) AS latest_2,
       MAX(LS3.sample_date) AS latest_3
  FROM LoadSample LS0
           LEFT OUTER JOIN LoadSample LS1
             ON LS1.sample_date < LS0.sample_date
             LEFT OUTER JOIN LoadSample LS2
               ON LS2.sample_date < LS1.sample_date
               LEFT OUTER JOIN LoadSample LS3
                 ON LS3.sample_date < LS2.sample_date
 GROUP BY LS0.sample_date;

--OracleのLAG
SELECT sample_date,
       LAG(sample_date, 1) OVER (ORDER BY sample_date) AS latest,
       LAG(sample_date, 2) OVER (ORDER BY sample_date) AS latest_2,
       LAG(sample_date, 3) OVER (ORDER BY sample_date) AS latest_3
  FROM LoadSample;


--リスト9 現在と過去の直近を求める：ウィンドウ関数版
SELECT MIN(sample_date)
           OVER (ORDER BY sample_date ASC
                 ROWS BETWEEN 1 PRECEDING AND 1 PRECEDING) AS past,
       sample_date AS cur_date,
       MAX(sample_date)
           OVER (ORDER BY sample_date DESC
                 ROWS BETWEEN 1 PRECEDING AND 1 PRECEDING) AS future
  FROM LoadSample;

--リスト10 現在と過去の直近を求める：自己結合版
SELECT MAX(LS1.sample_date) AS past,
       LS.sample_date AS sample_date,
       MIN(LS2.sample_date) AS future
  FROM LoadSample LS
       LEFT OUTER JOIN LoadSample LS1
         ON LS1.sample_date < LS.sample_date
         LEFT OUTER JOIN LoadSample LS2
           ON LS2.sample_date > LS.sample_date
 GROUP BY LS.sample_date;


--***********************************
--
--○2－3．小分けにしたグループ内での行間比較
--
--***********************************


CREATE TABLE LoadSample2
(machine     CHAR(3) NOT NULL,
 sample_date DATE NOT NULL , 
 load_amt    INTEGER NOT NULL , 
   PRIMARY KEY (machine, sample_date) ) ;

INSERT INTO LoadSample2 VALUES('PC1',  '2014-02-01', 1024);
INSERT INTO LoadSample2 VALUES('PC1',  '2014-02-02', 2366);
INSERT INTO LoadSample2 VALUES('PC1',  '2014-02-05', 2366);
INSERT INTO LoadSample2 VALUES('PC1',  '2014-02-07',  985);
INSERT INTO LoadSample2 VALUES('PC1',  '2014-02-08',  780);
INSERT INTO LoadSample2 VALUES('PC1',  '2014-02-12', 1000);
INSERT INTO LoadSample2 VALUES('PC2',  '2014-02-01',  999);
INSERT INTO LoadSample2 VALUES('PC2',  '2014-02-02',   50);
INSERT INTO LoadSample2 VALUES('PC2',  '2014-02-05',  328);
INSERT INTO LoadSample2 VALUES('PC2',  '2014-02-07',  913);
INSERT INTO LoadSample2 VALUES('PC3',  '2014-02-01', 2000);
INSERT INTO LoadSample2 VALUES('PC3',  '2014-02-02', 1000);


--リスト11 のクエリをそのまま実行してみる：うまくいかない
SELECT sample_date AS cur_date,
       MIN(sample_date)
           OVER (ORDER BY sample_date ASC
                 ROWS BETWEEN 1 PRECEDING AND 1 PRECEDING) AS latest_1,
       MIN(sample_date)
           OVER (ORDER BY sample_date ASC
                 ROWS BETWEEN 2 PRECEDING AND 2 PRECEDING) AS latest_2,
       MIN(sample_date)
           OVER (ORDER BY sample_date ASC
                 ROWS BETWEEN 3 PRECEDING AND 3 PRECEDING) AS latest_3
  FROM LoadSample2;


--リスト12 ウィンドウ関数版（machine列なし）
SELECT sample_date AS cur_date,
       MIN(sample_date) 
          OVER (ORDER BY sample_date ASC
                ROWS BETWEEN 1 PRECEDING AND 1 PRECEDING) AS latest_1,
       MIN(sample_date) 
          OVER (ORDER BY sample_date ASC
                ROWS BETWEEN 2 PRECEDING AND 2 PRECEDING) AS latest_2,
       MIN(sample_date) 
          OVER (ORDER BY sample_date ASC
                ROWS BETWEEN 3 PRECEDING AND 3 PRECEDING) AS latest_3
  FROM LoadSample2
 GROUP BY sample_date;



--ウィンドウ関数版（machine列あり）
SELECT machine,
       sample_date AS cur_date,
       MIN(sample_date) 
          OVER (PARTITION BY machine ORDER BY sample_date ASC
                ROWS BETWEEN 1 PRECEDING AND 1 PRECEDING) AS latest_1,
       MIN(sample_date) 
          OVER (PARTITION BY machine ORDER BY sample_date ASC
                ROWS BETWEEN 2 PRECEDING AND 2 PRECEDING) AS latest_2,
       MIN(sample_date) 
          OVER (PARTITION BY machine ORDER BY sample_date ASC
                ROWS BETWEEN 3 PRECEDING AND 3 PRECEDING) AS latest_3
  FROM LoadSample2
 GROUP BY machine, sample_date;


--リスト13 自己結合版（machine列なし）
SELECT LS0.sample_date AS cur_date,
       MAX(LS1.sample_date) AS latest_1,
       MAX(LS2.sample_date) AS latest_2,
       MAX(LS3.sample_date) AS latest_3
  FROM LoadSample2 LS0
           LEFT OUTER JOIN LoadSample2 LS1
             ON LS1.sample_date < LS0.sample_date
             LEFT OUTER JOIN LoadSample2 LS2
               ON LS2.sample_date < LS1.sample_date
               LEFT OUTER JOIN LoadSample2 LS3
                 ON LS3.sample_date < LS2.sample_date
 GROUP BY LS0.sample_date;


--リスト14 ウィンドウ関数版（machine列あり）
SELECT machine,
       sample_date AS cur_date,
       MIN(sample_date)
           OVER (PARTITION BY machine ORDER BY sample_date ASC
                 ROWS BETWEEN 1 PRECEDING AND 1 PRECEDING) AS latest_1,
       MIN(sample_date)
           OVER (PARTITION BY machine ORDER BY sample_date ASC
                 ROWS BETWEEN 2 PRECEDING AND 2 PRECEDING) AS latest_2,
       MIN(sample_date)
           OVER (PARTITION BY machine ORDER BY sample_date ASC
                 ROWS BETWEEN 3 PRECEDING AND 3 PRECEDING) AS latest_3
  FROM LoadSample2
 GROUP BY machine, sample_date;

--リスト15 自己結合版（machine列あり）
SELECT LS0.machine AS machine, 
       LS0.sample_date AS sample_date,
       MAX(LS1.sample_date) AS latest_1,
       MAX(LS2.sample_date) AS latest_2,
       MAX(LS3.sample_date) AS latest_3
  FROM LoadSample2 LS0
           LEFT OUTER JOIN LoadSample2 LS1
             ON LS1.sample_date < LS0.sample_date
             LEFT OUTER JOIN LoadSample2 LS2
               ON LS2.sample_date < LS1.sample_date
               LEFT OUTER JOIN LoadSample2 LS3
                 ON LS3.sample_date < LS2.sample_date
 GROUP BY LS0.machine, LS0.sample_date
 ORDER BY machine, sample_date;
